local NNGG = LibStub("AceAddon-3.0"):NewAddon("NoNoGoGo", "AceTimer-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local LW = LibStub("LibWindow-1.1")
local AC = LibStub("AceConfig-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ADB = LibStub("AceDB-3.0")

local match, sub, gsub = string.match, string.sub, string.gsub
local type, tonumber = type, tonumber
local InCombatLockdown, IsEncounterInProgress = InCombatLockdown, IsEncounterInProgress
local UnitIsGroupLeader, UnitIsGroupAssistant = UnitIsGroupLeader, UnitIsGroupAssistant
local LoggingCombat = LoggingCombat
local _,_,_,_,_,_,_,mapID = GetInstanceInfo()
local LastInstanceMapID = mapID
local pullTimerCount = nil
local pullTimerStarted = nil
local timedResetStarted = nil
local RWPullTimerCount = nil
local RWPullTimerStarted = nil
local currentPullTimerInterval = nil
local lastPullTimerInterval = nil
local testMode = nil
local statusBar = nil
local statusBarTimer = nil
local statusBarTestMode = nil
local loggingCombat = nil
local RWPullTimerMode = nil
local DBMPullTimerMode = nil
local BWPullTimerMode = nil
local resetPullTimerTicker = nil
local disableCombatLoggingTicker = nil
local addon = nil
local popupDialogShown = nil
local autoLogging = nil

local db
local defaults = {
   profile = {
      AnchorFrame = {},
		StatusBarFrame = {},
		AnchorWidth = 100,
		AnchorHeight = 150,
		Font = "Morpheus",
		FontColor = {r = 0, g = 0, b = 0, a = 1},
		FontOutline = "",
		FontSize = 14,
		Background = "Blizzard Marble",
		ButtonBackground = "Blizzard Parchment",
		BackgroundColor = {r = 1, g = 1, b = 1, a = 1},
		StatusBar = {
		   BarWidth = 250,
			BarHeight = 20,
			BarTexture = "Blizzard",
			BarColor = {r = 1, g = 0, b = 0, a = 1},
			BarBackgroundTexture = "Blizzard",
			BarBackgroundColor = {r = 0.0862745098039216, g = 0.0862745098039216, b = 0.0862745098039216, a = 1},
			Font = "Arial Narrow",
			FontColor = {r = 1, g = 1, b = 1, a = 1},
			FontOutline = "OUTLINE",
			FontSize = 14
		},
   },
   global = {
      PullTimerCount = 10,
      TestMode = false,
		StatusBar = false,
		StatusBarTestMode = false,
		CombatLogging = false,
		PullTimerMode = "RW"
   }
}

local function initializeLocalVars()
   pullTimerCount = NNGG.db.global.PullTimerCount
	testMode = NNGG.db.global.TestMode
   logCombat = NNGG.db.global.CombatLogging
   statusBar = NNGG.db.global.StatusBar
   statusBarTestMode = NNGG.db.global.StatusBarTestMode
end

local order = 0
local function getOrder()
	order = order + 1
	return order
end

local options = {
   name = "NoNoGoGo",
   handler = NNGG,
   type = "group",
	guiInline = true,
   args = {
		generalsettings = {
			order = getOrder(),
			type = "group",
			name = "General Settings",
			guiInline = true,
			args = {
            pulltimermode = {
					type = "select",
					order = getOrder(),
					name = "Pull Timer Mode",
					desc = "Set the pull timer mode.",
					values = {
	               ["RW"]  = "Raid Warning",
	               ["DBM"] = "Deadly Boss Mods",
	               ["BW"]  = "Big Wigs",
	            },
					set = function(info, value)
					   NNGG.db.global.PullTimerMode = value
						NNGG:setPullTimerMode(value)
					end,
					get = function(info) return NNGG.db.global.PullTimerMode end
				},
            anchorwidth = {
               order = getOrder(),
               type = "range",
               name = "Width",
               desc = "Set the width of the main frames.",
               min = 50, max = 200, step = 0.5,
               set = function(info, value)
                  NNGG.db.profile.AnchorWidth = value
                  NNGG.PT_f:SetWidth(NNGG.db.profile.AnchorWidth-10)
						NNGG.RC_f:SetWidth(NNGG.db.profile.AnchorWidth-10)
                  NNGG.A_f:SetWidth(NNGG.db.profile.AnchorWidth)
               end,
               get = function(info) return NNGG.db.profile.AnchorWidth end
            },
            anchorheight = {
               order = getOrder(),
               type = "range",
               name = "Height",
               desc = "Set the height of the main frames.",
               min = 50, max = 200, step = 0.5,
               set = function(info, value)
                  NNGG.db.profile.AnchorHeight = value
                  NNGG.PT_f:SetHeight((NNGG.db.profile.AnchorHeight/2)-7.5)
						NNGG.RC_f:SetHeight((NNGG.db.profile.AnchorHeight/2)-7.5)
                  NNGG.A_f:SetHeight(NNGG.db.profile.AnchorHeight)
               end,
               get = function(info) return NNGG.db.profile.AnchorHeight end
            },
            combatlogging = {
               order = getOrder(),
               type = "toggle",
               name = "Combat Logging",
               desc = "Toggle automatic combat logging.",
               set = function(info, value)
					   NNGG.db.global.CombatLogging = value
					   logCombat = value
					end,
               get = function(info) return NNGG.db.global.CombatLogging end
            },
            testmode = {
               order = getOrder(),
               type = "toggle",
               name = "Test Mode",
               desc = "Toggle test mode.",
               set = function(info, value)
					   NNGG.db.global.TestMode = value
                  testMode = value
						NNGG:setTestMode(value)
					end,
               get = function(info) return NNGG.db.global.TestMode end
            },
            statusbar = {
               order = getOrder(),
               type = "toggle",
               name = "Status Bar",
               desc = "Toggle status bar.",
               set = function(info, value)
					   NNGG.db.global.StatusBar = value
						statusBar = value
					end,
               get = function(info) return NNGG.db.global.StatusBar end
            },
            statusbartestmode = {
               order = getOrder(),
               type = "toggle",
               name = "Status Bar Test Mode",
               desc = "Toggle status bar test mode.",
               set = function(info, value)
		      	   NNGG.db.global.StatusBarTestMode = value
						statusBarTestMode = value
		      		NNGG:setStatusBarMode()
		      	end,
               get = function(info) return NNGG.db.global.StatusBarTestMode end
            },
				font = {
					type = "select", dialogControl = 'LSM30_Font',
					order = getOrder(),
					name = "Font",
					desc = "Set the font of the button text.",
					values = AceGUIWidgetLSMlists.font,
					set = function(info, value)
					   NNGG.db.profile.Font = value
						NNGG.PT_f.pullTimerText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
						NNGG.RC_f.readyCheckText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.Font end
				},
				fontoutline = {
					type = "select",
					order = getOrder(),
					name = "Font Outline",
					desc = "Set the font outline of the button text.",
					values = {
	               [""]             = "None",
	               ["OUTLINE"]      = "Outline",
	               ["THICKOUTLINE"] = "Thick Outline",
	            },
					set = function(info, value)
					   NNGG.db.profile.FontOutline = value
						NNGG.PT_f.pullTimerText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
						NNGG.RC_f.readyCheckText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.FontOutline end
				},
				fontcolor = {
					type = "color",
					order = getOrder(),
					name = "Font Color",
					desc = "Set the font color of the button text.",
					hasAlpha = true,
					set = function(info, r, g, b, a)
					   NNGG.db.profile.FontColor.r, NNGG.db.profile.FontColor.g, NNGG.db.profile.FontColor.b, NNGG.db.profile.FontColor.a = r, g, b, a
						NNGG.PT_f.pullTimerText:SetTextColor(r, g, b, a)
						NNGG.RC_f.readyCheckText:SetTextColor(r, g, b, a)
					end,
					get = function(info) return NNGG.db.profile.FontColor.r, NNGG.db.profile.FontColor.g, NNGG.db.profile.FontColor.b, NNGG.db.profile.FontColor.a end,
				},
				fontsize = {
					type = "range",
					order = getOrder(),
					name = "Font Size",
					desc = "Set the font size of the button text.",
					min = 6, max = 25, step = 1,
					set = function(info, value)
					   NNGG.db.profile.FontSize = value
						NNGG.PT_f.pullTimerText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
						NNGG.RC_f.readyCheckText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.FontSize end
				},
				texture = {
					type = "select", dialogControl = 'LSM30_Background',
					order = getOrder(),
					name = "Backdrop Texture",
					desc = "Set the texture for the backdrop of the main frames.",
					values = AceGUIWidgetLSMlists.background,
					set = function(info, value)
						NNGG.db.profile.Background = value
						NNGG.A_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.Background))
					end,
					get = function(info) return NNGG.db.profile.Background end
				},
				backgroundcolor = {
					type = "color",
					order = getOrder(),
					name = "Frame Color",
					desc = "Set the color for the main frames.",
					hasAlpha = true,
					set = function(info, r, g, b, a)
						NNGG.db.profile.BackgroundColor.r, NNGG.db.profile.BackgroundColor.g, NNGG.db.profile.BackgroundColor.b, NNGG.db.profile.BackgroundColor.a = r, g, b, a
                  NNGG.A_t:SetVertexColor(r, g, b, a)
						NNGG.PT_t:SetVertexColor(r, g, b, a)
						NNGG.RC_t:SetVertexColor(r, g, b, a)
					end,
					get = function(info) return NNGG.db.profile.BackgroundColor.r, NNGG.db.profile.BackgroundColor.g, NNGG.db.profile.BackgroundColor.b, NNGG.db.profile.BackgroundColor.a end
				},
				buttontexture = {
					type = "select", dialogControl = 'LSM30_Background',
					order = getOrder(),
					name = "Button Texture",
					desc = "Set the texture for the buttons.",
					values = AceGUIWidgetLSMlists.background,
					set = function(info, value)
						NNGG.db.profile.ButtonBackground = value
						NNGG.PT_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.ButtonBackground))
						NNGG.RC_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.ButtonBackground))
					end,
					get = function(info) return NNGG.db.profile.ButtonBackground end
				}
			}
		},
		pulltimerstatusbarsettings = {
			order = 2,
			type = "group",
			name = "Status Bar Settings",
			guiInline = true,
			args = {
				FontSize = {
					type = "range",
					order = getOrder(),
					name = "Font Size",
					desc = "Set the font size of the status bar text.",
					min = 6, max = 25, step = 1,
					set = function(info, value)
					   NNGG.db.profile.StatusBar.FontSize = value
						NNGG.SB_f.pulltext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
						NNGG.SB_f.timertext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.FontSize end
				},
            width = {
               order = getOrder(),
               type = "range",
               name = "Width",
               desc = "Set the width of the status bar.",
               min = 1, max = 500, step = 0.1,
               set = function(info, value)
                  NNGG.db.profile.StatusBar.BarWidth = value
                  NNGG.SB_f:SetWidth(NNGG.db.profile.StatusBar.BarWidth)
               end,
               get = function(info) return NNGG.db.profile.StatusBar.BarWidth end
            },
            height = {
               order = getOrder(),
               type = "range",
               name = "Height",
               desc = "Set the height of the status bar.",
               min = 1, max = 50, step = 0.1,
               set = function(info, value)
                  NNGG.db.profile.StatusBar.BarHeight = value
                  NNGG.SB_f:SetHeight(NNGG.db.profile.StatusBar.BarHeight)
               end,
               get = function(info) return NNGG.db.profile.StatusBar.BarHeight end
            },
				font = {
					type = "select", dialogControl = 'LSM30_Font',
					order = getOrder(),
					name = "Font",
					desc = "Set the font of the status bar text.",
					values = AceGUIWidgetLSMlists.font,
					set = function(info, value)
					   NNGG.db.profile.StatusBar.Font = value
						NNGG.SB_f.pulltext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
						NNGG.SB_f.timertext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.Font end
				},
				Texture = {
					type = "select", dialogControl = 'LSM30_Statusbar',
					order = getOrder(),
					name = "Bar Texture",
					desc = "Set the texture for the status bar.",
					values = AceGUIWidgetLSMlists.statusbar,
					set = function(info, value)
						NNGG.db.profile.StatusBar.BarTexture = value
						NNGG.SB_f:SetStatusBarTexture(LSM:Fetch("statusbar", NNGG.db.profile.StatusBar.BarTexture))
					end,
					get = function(info) return NNGG.db.profile.StatusBar.BarTexture end
				},
				backgroundtexture = {
					type = "select", dialogControl = 'LSM30_Statusbar',
					order = getOrder(),
					name = "Background Texture",
					desc = "Set the background texture for the status bar.",
					values = AceGUIWidgetLSMlists.statusbar,
					set = function(info, value)
						NNGG.db.profile.StatusBar.BarBackgroundTexture = value
						NNGG.SB_t:SetTexture(LSM:Fetch("statusbar", NNGG.db.profile.StatusBar.BarBackgroundTexture))
					end,
					get = function(info) return NNGG.db.profile.StatusBar.BarBackgroundTexture end
				},
				FontColor = {
					type = "color",
					order = getOrder(),
					name = "Font Color",
					desc = "Set the font color of the status bar text.",
					hasAlpha = true,
					set = function(info, r, g, b, a)
					   NNGG.db.profile.StatusBar.FontColor.r, NNGG.db.profile.StatusBar.FontColor.g, NNGG.db.profile.StatusBar.FontColor.b, NNGG.db.profile.StatusBar.FontColor.a = r, g, b, a
						NNGG.SB_f.pulltext:SetTextColor(r, g, b, a)
						NNGG.SB_f.timertext:SetTextColor(r, g, b, a)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.FontColor.r, NNGG.db.profile.StatusBar.FontColor.g, NNGG.db.profile.StatusBar.FontColor.b, NNGG.db.profile.StatusBar.FontColor.a end,
				},
				color = {
					type = "color",
					order = getOrder(),
					name = "Bar Color",
					desc = "Set the color of the status bar",
					hasAlpha = true,
					set = function(info, r, g, b, a)
						NNGG.db.profile.StatusBar.BarColor.r, NNGG.db.profile.StatusBar.BarColor.g, NNGG.db.profile.StatusBar.BarColor.b, NNGG.db.profile.StatusBar.BarColor.a = r, g, b, a
						NNGG.SB_f:SetStatusBarColor(r, g, b, a)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.BarColor.r, NNGG.db.profile.StatusBar.BarColor.g, NNGG.db.profile.StatusBar.BarColor.b, NNGG.db.profile.StatusBar.BarColor.a end
				},
				backgroundcolor = {
					type = "color",
					order = getOrder(),
					name = "Background Color",
					desc = "Set the background color of the status bar.",
					hasAlpha = true,
					set = function(info, r, g, b, a)
						NNGG.db.profile.StatusBar.BarBackgroundColor.r, NNGG.db.profile.StatusBar.BarBackgroundColor.g, NNGG.db.profile.StatusBar.BarBackgroundColor.b, NNGG.db.profile.StatusBar.BarBackgroundColor.a = r, g, b, a
                  NNGG.SB_t:SetVertexColor(r, g, b, a)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.BarBackgroundColor.r, NNGG.db.profile.StatusBar.BarBackgroundColor.g, NNGG.db.profile.StatusBar.BarBackgroundColor.b, NNGG.db.profile.StatusBar.BarBackgroundColor.a end
				},
				FontOutline = {
					type = "select",
					order = getOrder(),
					name = "Font Outline",
					desc = "Set the font outline of the status bar text.",
					values = {
	               [""]             = "None",
	               ["OUTLINE"]      = "Outline",
	               ["THICKOUTLINE"] = "Thick Outline",
	            },
					set = function(info, value)
					   NNGG.db.profile.StatusBar.FontOutline = value
						NNGG.SB_f.pulltext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
						NNGG.SB_f.timertext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
					end,
					get = function(info) return NNGG.db.profile.StatusBar.FontOutline end
				},
			}
		}
   }
}

function NNGG:toggleFrames(toggle)
   if toggle then
	   NNGG.A_f:Show()
	   NNGG.RC_f:Show()
      NNGG.PT_f:Show()
	else
	   NNGG.A_f:Hide()
	   NNGG.RC_f:Hide()
      NNGG.PT_f:Hide()
	end
end

function NNGG:setPullTimerMode(mode)
	if mode == "DBM" then
      RWPullTimerMode = false
      DBMPullTimerMode = true
      BWPullTimerMode = false
	elseif mode == "BW" then
      RWPullTimerMode = false
      DBMPullTimerMode = false
      BWPullTimerMode = true
	else
      RWPullTimerMode = true
      DBMPullTimerMode = false
      BWPullTimerMode = false
	end
end

local function isLeadOrAssist()
   if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
      return true
   end
   return false
end

local function isRaidWarningEligible()
   if UnitInRaid("player") and isLeadOrAssist() then
      return true
   end
   return false
end

local function inCombat()
   if not InCombatLockdown() and not IsEncounterInProgress() then
	   return false
	end
	return true
end

function NNGG:setTestMode(enable)
   if enable then
	   NNGG:toggleFrames(true)
   else
      if inCombat() or not isLeadOrAssist() then
		   NNGG:toggleFrames(false)
		end
   end
end

local function statusBarOnUpdate(self)
	local timeLeft = statusBarTimer - GetTime()
	if timeLeft and timeLeft > 0 then
		NNGG.SB_f:SetValue(timeLeft)
		NNGG.SB_f.pulltext:SetFormattedText("Pull in:")
		NNGG.SB_f.timertext:SetFormattedText("%.1f", timeLeft)
	else
	   NNGG:setStatusBarMode()
	end
end

function NNGG:initiateStatusBar(timer)
   if not tonumber(timer) then return end
	NNGG.SB_f:Show()
	NNGG.SB_f:SetMinMaxValues(0, timer)
	statusBarTimer = GetTime() + timer
	NNGG.SB_f:SetScript("OnUpdate", statusBarOnUpdate)
end

function NNGG:setStatusBarMode()
   if statusBarTestMode then
		NNGG:initiateStatusBar(12)
   else
	   NNGG.SB_f:SetScript("OnUpdate", nil)
      NNGG.SB_f:Hide()
	end
end

local function sendSync(prefix, msg)
	msg = msg or ""
	if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and IsInInstance() and not C_Garrison:IsOnGarrisonMap() then
		SendAddonMessage("NoNoGoGo", prefix .. "\t" .. msg, "INSTANCE_CHAT")
	else
		if UnitInRaid("player") then
			SendAddonMessage("NoNoGoGo", prefix .. "\t" .. msg, "RAID")
		elseif IsInGroup(LE_PARTY_CATEGORY_HOME) then
			SendAddonMessage("NoNoGoGo", prefix .. "\t" .. msg, "PARTY")
		end
	end
end

function NNGG:enableCombatLogging()
	if logCombat and not autoLogging then
	   print("|cffDA2820NoNoGoGo|r|cffFFFF00: Combat logging |cff77C8FFenabled|r|cffFFFF00.|r")
	   LoggingCombat(true)
		autoLogging = true
	end
end

function NNGG:disableCombatLogging()
	NNGG:CancelTimer(disableCombatLoggingTicker)
	disableCombatLoggingTicker = nil
	if logCombat and autoLogging and not inCombat() then
	   print("|cffDA2820NoNoGoGo|r|cffFFFF00: Combat logging |cff77C8FFdisabled|r|cffFFFF00.|r")
	   LoggingCombat(false)
		autoLogging = false
	end
end

function NNGG:resetPullTimer()
   if not timedResetStarted then return end
	NNGG.PT_f.pullTimerText:SetFormattedText("Pull Timer")
	NNGG:CancelTimer(resetPullTimerTicker)
	resetPullTimerTicker = nil
	addon = nil
	pullTimerStarted = false
	timedResetStarted = false
end

function NNGG:startTimedReset(seconds)
   if timedResetStarted then return end
   NNGG.PT_f.pullTimerText:SetFormattedText("Cancel")
	NNGG:CancelTimer(disableCombatLoggingTicker)
	disableCombatLoggingTicker = nil
	disableCombatLoggingTicker = NNGG:ScheduleTimer("disableCombatLogging", seconds)
	resetPullTimerTicker = NNGG:ScheduleTimer("resetPullTimer", seconds)
	pullTimerStarted = true
	timedResetStarted = true
end

local function handleMessage(timer)
   if not tonumber(timer) then return end
	local timer = tonumber(match(timer, "%d+"))
	if timer > 0 then
	   NNGG:startTimedReset(timer + 5)
		NNGG:enableCombatLogging()
	else
	   NNGG:resetPullTimer()
		NNGG:disableCombatLogging()
	end
end

function NNGG:resetRWPullTimer()
   NNGG.RWPT_f:SetScript("OnUpdate", nil)
   RWPullTimerStarted = nil
	currentPullTimerInterval = nil
   lastPullTimerInterval = 0
end

local function updateRWPullTimer(self, elapsed)
   local messg
   if not RWPullTimerStarted then
      messg = gsub("Pulling in $c sec.", "$c", RWPullTimerCount)
      SendChatMessage(messg, "RAID_WARNING")
		RWPullTimerStarted = true
		currentPullTimerInterval = currentPullTimerInterval - 1
   end
   lastPullTimerInterval = lastPullTimerInterval + elapsed
   if lastPullTimerInterval >= 1 then
      lastPullTimerInterval = 0
      if currentPullTimerInterval > 0 then
         messg = gsub(currentPullTimerInterval, "", "")
         SendChatMessage(messg, "RAID_WARNING")
         currentPullTimerInterval = currentPullTimerInterval - 1
      else
         SendChatMessage("Pull now!", "RAID_WARNING")
         NNGG:resetRWPullTimer()
      end
   end
end

local function SBOnEvent(self, event, prefix, message, channel, sender)
   if not statusBar then return end
   if prefix == "D4" and sub(message, 1, 2) == "PT" then
   	local _, timer = strsplit("\t", message)
	   NNGG:initiateStatusBar(timer)
	elseif prefix == "BigWigs" then
		local bwPrefix, bwMsg = message:match("^(%u-):(.+)")
		if bwPrefix == "T" then
			local _, timer = strsplit("", bwMsg)
	      NNGG:initiateStatusBar(timer)
		end
	   NNGG:initiateStatusBar(timer)
	elseif prefix == "NoNoGoGo" and sub(message, 1, 2) == "PT" then
	   local _, timer = strsplit("\t", message)
	   NNGG:initiateStatusBar(timer)
   end
end

local function PTOnEvent(self, event, prefix, message, channel, sender)
   if prefix == "D4" and sub(message, 1, 2) == "PT" then
   	local _, timer = strsplit("\t", message)
		addon = "DBM"
	   handleMessage(timer)
	elseif prefix == "BigWigs" then
		local bwPrefix, bwMsg = message:match("^(%u-):(.+)")
		if bwPrefix == "T" then
			local _, timer = strsplit("", bwMsg)
			addon = "BW"
	      handleMessage(timer)
		end
	elseif prefix == "NoNoGoGo" and sub(message, 1, 2) == "PT" then
	   local _, timer = strsplit("\t", message)
		addon = "NNGG"
	   handleMessage(timer)
	end
end

local function onPlayerEnteringWorld()
   if testMode then return end
   if not inCombat() and isLeadOrAssist() then
	   NNGG:toggleFrames(true)
	else
	   NNGG:toggleFrames(false)
	end
end

local function onPlayerRoleAssigned()
   if testMode then return end
	if not inCombat() and isLeadOrAssist() then
	   NNGG:toggleFrames(true)
	else
	   NNGG:toggleFrames(false)
	end
end

local function onPlayerRegenDisabled()
	NNGG:resetPullTimer()
	if not testMode then
	   NNGG:toggleFrames(false)
	end
end

local function onPlayerRegenEnabled()
	disableCombatLoggingTicker = NNGG:ScheduleTimer("disableCombatLogging", 6)
   if not inCombat() and isLeadOrAssist() then
	   NNGG:toggleFrames(true)
	end
end

local function onEncounterEnd()
   disableCombatLoggingTicker = NNGG:ScheduleTimer("disableCombatLogging", 6)
   if isLeadOrAssist() then
	   NNGG:toggleFrames(true)
   end
end

local function PTOnMouseDown(self, button)
   if button == "LeftButton" and not IsAltKeyDown() then
	   if not pullTimerStarted and not timedResetStarted then
         if DBMPullTimerMode and IsAddOnLoaded("DBM-Core") and isLeadOrAssist() then
            SlashCmdList.DEADLYBOSSMODS("pull ".. pullTimerCount)
				pullTimerStarted = true
         elseif BWPullTimerMode and IsAddOnLoaded("BigWigs_Core") and isLeadOrAssist() then
            SlashCmdList.BIGWIGSPULL("".. pullTimerCount)
				pullTimerStarted = true
         elseif RWPullTimerMode and IsAddOnLoaded("NoNoGoGo") and isRaidWarningEligible() then
            SlashCmdList.NONOGOGO("pull ".. pullTimerCount)
				pullTimerStarted = true
			end
		elseif pullTimerStarted and timedResetStarted then
         if addon == "DBM" or addon == "BW" then
			   if IsAddOnLoaded("DBM-Core") then
               SlashCmdList.DEADLYBOSSMODS("pull 0")
			      pullTimerStarted = false
				elseif IsAddOnLoaded("BigWigs_Core") then
				   SlashCmdList.BIGWIGSPULL("0")
			      pullTimerStarted = false
            end
         elseif addon == "NNGG" and IsAddOnLoaded("NoNoGoGo") then
            SlashCmdList.NONOGOGO("pull 0")
			   pullTimerStarted = false
			end
      end
   elseif button == "RightButton" then
	   if not popupDialogShown then
	      StaticPopup_Show("SET_PULL_TIMER")
			popupDialogShown = true
		else
		   StaticPopup_Hide("SET_PULL_TIMER")
		   popupDialogShown = false
		end
   end
end

local function RCOnMouseDown(self, button)
   if button == "LeftButton" and not IsAltKeyDown() then
		DoReadyCheck()
	end
end

local function onAccept(self, seconds)
   popupDialogShown = false
	seconds = tonumber(seconds) or 10
	if seconds >= 1 and seconds <=60 then
	   NNGG.db.global.PullTimerCount = seconds
		pullTimerCount = seconds
		print("|cffDA2820NoNoGoGo|r|cffFFFF00: Pull Timer set to:|r|cff77C8FF "..seconds.."|r |cff77C8FFseconds|r|cffFFFF00.|r")
	else
	   print("|cffDA2820NoNoGoGo|r|cffFFFF00: The global pull timer must be between 1 and 60 seconds.|r")
	end
end

StaticPopupDialogs["SET_PULL_TIMER"] =
{
   text = "Set pull timer.",
   button1 = "Set",
   button2 = "Cancel",
   OnAccept = function(self)
	   onAccept(self, self.editBox:GetText())
   end,
	OnCancel = function()
	   popupDialogShown = false
	end,
	EditBoxOnEscapePressed = function(self)
		popupDialogShown = false
		self:GetParent():Hide()
   end,
	EditBoxOnEnterPressed = function(self)
	   onAccept(self, self:GetParent().editBox:GetText())
      self:GetParent():Hide()
   end,
	maxLetters = 2,
   preferredIndex = 3,
	hasEditBox = true,
   whileDead = true,
}

local function RWPullCommand(seconds)
	RWPullTimerCount = seconds
	currentPullTimerInterval = RWPullTimerCount
	sendSync("PT", RWPullTimerCount.."\t"..LastInstanceMapID)
	if RWPullTimerCount > 0 then
	   NNGG.RWPT_f:SetScript("OnUpdate", updateRWPullTimer)
	else
	   NNGG:resetRWPullTimer()
	end
end

SLASH_NONOGOGO1 = "/nngg"
SlashCmdList["NONOGOGO"] = function(msg)
   local cmd = msg:lower()
   if cmd == "" then
      InterfaceOptionsFrame_OpenToCategory("NoNoGoGo")
		InterfaceOptionsFrame_OpenToCategory("NoNoGoGo")
		InterfaceOptionsFrame_OpenToCategory("NoNoGoGo")
   elseif cmd:sub(1, 4) == "pull" then
      if not isRaidWarningEligible() then return end
      local seconds = tonumber(cmd:sub(5)) or 5
	   if seconds >=0 and seconds <=10 then
		   RWPullCommand(seconds)
	   else
		   RWPullCommand(5)
	   end
	end
end

function NNGG:createFrames()
	-- Anchor frame
	NNGG.A_f = CreateFrame("Frame", "NNGGAnchorFrame", UIParent)
   LW.RegisterConfig(NNGG.A_f, NNGG.db.profile.AnchorFrame)
   LW.RestorePosition(NNGG.A_f)
   LW.MakeDraggable(NNGG.A_f)
	LW.EnableMouseOnAlt(NNGG.A_f)
   NNGG.A_f:SetFrameStrata("BACKGROUND")
   NNGG.A_f:SetWidth(NNGG.db.profile.AnchorWidth)
   NNGG.A_f:SetHeight(NNGG.db.profile.AnchorHeight)
   NNGG.A_f:SetPoint("CENTER")
	NNGG.A_f:SetClampedToScreen(true)
   NNGG.A_f:RegisterForDrag("LeftButton")
	
   NNGG.A_t = NNGG.A_f:CreateTexture(nil, "BACKGROUND")
   NNGG.A_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.Background))
   NNGG.A_t:SetAllPoints(true)
   NNGG.A_t:SetVertexColor(NNGG.db.profile.BackgroundColor.r, NNGG.db.profile.BackgroundColor.g, NNGG.db.profile.BackgroundColor.b, NNGG.db.profile.BackgroundColor.a)
	
	-- Ready Check frame
   NNGG.RC_f = CreateFrame("Button", "NNGGReadyCheckFrame", UIParent)
   NNGG.RC_f:SetFrameStrata("LOW")
   NNGG.RC_f:SetWidth(NNGG.db.profile.AnchorWidth-10)
   NNGG.RC_f:SetHeight((NNGG.db.profile.AnchorHeight/2)-7.5)
   NNGG.RC_f:SetPoint("BOTTOM", NNGG.A_f, "BOTTOM", 0, 5)
   NNGG.RC_f:RegisterForClicks("LeftButtonDown")
   NNGG.RC_f:SetClampedToScreen(true)
   NNGG.RC_f.readyCheckText = NNGG.RC_f:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
   NNGG.RC_f.readyCheckText:SetPoint("CENTER", NNGG.RC_f)
	NNGG.RC_f.readyCheckText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
	NNGG.RC_f.readyCheckText:SetTextColor(NNGG.db.profile.FontColor.r, NNGG.db.profile.FontColor.g, NNGG.db.profile.FontColor.b, NNGG.db.profile.FontColor.a)
   NNGG.RC_f.readyCheckText:SetFormattedText("Ready Check")
   NNGG.RC_f:SetScript("OnMouseDown", RCOnMouseDown)
	NNGG.RC_f:SetHighlightTexture("interface\\addons\\NoNoGoGo\\textureHighlight.tga")
	
   NNGG.RC_t = NNGG.RC_f:CreateTexture(nil, "BACKGROUND")
   NNGG.RC_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.ButtonBackground))
   NNGG.RC_t:SetAllPoints(true)
   NNGG.RC_t:SetVertexColor(NNGG.db.profile.BackgroundColor.r, NNGG.db.profile.BackgroundColor.g, NNGG.db.profile.BackgroundColor.b, NNGG.db.profile.BackgroundColor.a)
	
   -- Pull Timer frame
   NNGG.PT_f = CreateFrame("Button", "NNGGPullTimerFrame", UIParent)
   NNGG.PT_f:SetFrameStrata("LOW")
   NNGG.PT_f:SetWidth(NNGG.db.profile.AnchorWidth-10)
   NNGG.PT_f:SetHeight((NNGG.db.profile.AnchorHeight/2)-7.5)
   NNGG.PT_f:SetPoint("TOP", NNGG.A_f, "TOP", 0, -5)
   NNGG.PT_f:RegisterForClicks("LeftButtonDown")
   NNGG.PT_f:SetClampedToScreen(true)
   NNGG.PT_f.pullTimerText = NNGG.PT_f:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
   NNGG.PT_f.pullTimerText:SetPoint("CENTER", NNGG.PT_f)
	NNGG.PT_f.pullTimerText:SetFont(LSM:Fetch("font", NNGG.db.profile.Font), NNGG.db.profile.FontSize, NNGG.db.profile.FontOutline)
	NNGG.PT_f.pullTimerText:SetTextColor(NNGG.db.profile.FontColor.r, NNGG.db.profile.FontColor.g, NNGG.db.profile.FontColor.b, NNGG.db.profile.FontColor.a)
   NNGG.PT_f.pullTimerText:SetFormattedText("Pull Timer")
   NNGG.PT_f:SetScript("OnMouseDown", PTOnMouseDown)
	NNGG.PT_f:SetHighlightTexture("interface\\addons\\NoNoGoGo\\textureHighlight.tga")
		
   NNGG.PT_t = NNGG.PT_f:CreateTexture(nil, "BACKGROUND")
   NNGG.PT_t:SetTexture(LSM:Fetch("background", NNGG.db.profile.ButtonBackground))
   NNGG.PT_t:SetAllPoints(true)
   NNGG.PT_t:SetVertexColor(NNGG.db.profile.BackgroundColor.r, NNGG.db.profile.BackgroundColor.g, NNGG.db.profile.BackgroundColor.b, NNGG.db.profile.BackgroundColor.a)
	
	-- Pull Timer Event frame
   local PTE_f = CreateFrame("Frame")
   PTE_f:RegisterEvent("CHAT_MSG_ADDON")
   PTE_f:SetScript("OnEvent", PTOnEvent)
	
   -- Raid warning pull timer frame
   NNGG.RWPT_f = CreateFrame("Frame")

   -- Status Bar frame
   NNGG.SB_f = CreateFrame("StatusBar", "NNGGStatusBarFrame", UIParent)
   LW.RegisterConfig(NNGG.SB_f, NNGG.db.profile.StatusBarFrame)
   LW.RestorePosition(NNGG.SB_f)
   LW.MakeDraggable(NNGG.SB_f)
	LW.EnableMouseOnAlt(NNGG.SB_f)
   NNGG.SB_f:SetWidth(NNGG.db.profile.StatusBar.BarWidth)
   NNGG.SB_f:SetHeight(NNGG.db.profile.StatusBar.BarHeight)
   NNGG.SB_f:SetPoint("CENTER")
	NNGG.SB_f:SetClampedToScreen(true)
   NNGG.SB_f.pulltext = NNGG.SB_f:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
   NNGG.SB_f.pulltext:SetPoint("LEFT", NNGG.SB_f, "LEFT", 5, 0)
	NNGG.SB_f.pulltext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
	NNGG.SB_f.pulltext:SetTextColor(NNGG.db.profile.StatusBar.FontColor.r, NNGG.db.profile.StatusBar.FontColor.g, NNGG.db.profile.StatusBar.FontColor.b, NNGG.db.profile.StatusBar.FontColor.a)
   NNGG.SB_f.timertext = NNGG.SB_f:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
   NNGG.SB_f.timertext:SetPoint("RIGHT", NNGG.SB_f, "RIGHT", -5, 0)
	NNGG.SB_f.timertext:SetFont(LSM:Fetch("font", NNGG.db.profile.StatusBar.Font), NNGG.db.profile.StatusBar.FontSize, NNGG.db.profile.StatusBar.FontOutline)
	NNGG.SB_f.timertext:SetTextColor(NNGG.db.profile.StatusBar.FontColor.r, NNGG.db.profile.StatusBar.FontColor.g, NNGG.db.profile.StatusBar.FontColor.b, NNGG.db.profile.StatusBar.FontColor.a)
	NNGG.SB_f:SetStatusBarTexture(LSM:Fetch("statusbar", NNGG.db.profile.StatusBar.BarTexture))
	NNGG.SB_f:SetStatusBarColor(NNGG.db.profile.StatusBar.BarColor.r, NNGG.db.profile.StatusBar.BarColor.g, NNGG.db.profile.StatusBar.BarColor.b, NNGG.db.profile.StatusBar.BarColor.a)
   NNGG:setStatusBarMode()
	
   NNGG.SB_t = NNGG.SB_f:CreateTexture(nil, "BACKGROUND")
   NNGG.SB_t:SetTexture(LSM:Fetch("statusbar", NNGG.db.profile.StatusBar.BarBackgroundTexture))
   NNGG.SB_t:SetAllPoints(true)
   NNGG.SB_t:SetVertexColor(NNGG.db.profile.StatusBar.BarBackgroundColor.r, NNGG.db.profile.StatusBar.BarBackgroundColor.g, NNGG.db.profile.StatusBar.BarBackgroundColor.b, NNGG.db.profile.StatusBar.BarBackgroundColor.a)
   
	-- Status Bar Event frame
	local SBE_f = CreateFrame("Frame")
   SBE_f:RegisterEvent("CHAT_MSG_ADDON")
   SBE_f:SetScript("OnEvent", SBOnEvent)
end

function NNGG:OnInitialize()
   print("|cffDA2820NoNoGoGo|r|cffFFFF00:|r |cff00FF00/nngg|r |cffFFFF00for options.|r")
	NNGG.db = ADB:New("NoNoGoGoDB", defaults)
   AC:RegisterOptionsTable("NoNoGoGo", options)
   ACD:AddToBlizOptions("NoNoGoGo")
	initializeLocalVars()
	RegisterAddonMessagePrefix("D4")
   RegisterAddonMessagePrefix("BigWigs")
	RegisterAddonMessagePrefix("NoNoGoGo")
   NNGG:RegisterEvent("PLAYER_REGEN_DISABLED", onPlayerRegenDisabled)
   NNGG:RegisterEvent("PLAYER_REGEN_ENABLED", onPlayerRegenEnabled)
   NNGG:RegisterEvent("PLAYER_ENTERING_WORLD", onPlayerEnteringWorld)
   NNGG:RegisterEvent("PLAYER_ROLES_ASSIGNED", onPlayerRoleAssigned)
	NNGG:RegisterEvent("ENCOUNTER_END", onEncounterEnd)
   NNGG:createFrames()
	NNGG:resetRWPullTimer()
	NNGG:setPullTimerMode(NNGG.db.global.PullTimerMode)
end